@@ -5,17 +5,12 @@ use test_case::test_case;
 
 use crypto_crawler::*;
 use crypto_markets::MarketType;
-use std::thread_local;
-use std::{
-    cell::RefCell,
-    sync::{Arc, Mutex},
-};
 
 const EXCHANGE_NAME: &str = "kraken";
 
 #[test_case(MarketType::Spot, "XBT/USD")]
 fn test_crawl_trade(market_type: MarketType, symbol: &str) {
-    gen_test_code!(
+    test_one_symbol!(
         crawl_trade,
         EXCHANGE_NAME,
         market_type,
@@ -26,7 +21,7 @@ fn test_crawl_trade(market_type: MarketType, symbol: &str) {
 
 #[test_case(MarketType::Spot, "XBT/USD")]
 fn test_crawl_l2_event(market_type: MarketType, symbol: &str) {
-    gen_test_code!(
+    test_one_symbol!(
         crawl_l2_event,
         EXCHANGE_NAME,
         market_type,
@@ -37,7 +32,7 @@ fn test_crawl_l2_event(market_type: MarketType, symbol: &str) {
 
 #[test_case(MarketType::Spot, "XBT/USD")]
 fn test_crawl_bbo(market_type: MarketType, symbol: &str) {
-    gen_test_code!(
+    test_one_symbol!(
         crawl_bbo,
         EXCHANGE_NAME,
         market_type,
@@ -48,7 +43,7 @@ fn test_crawl_bbo(market_type: MarketType, symbol: &str) {
 
 #[test_case(MarketType::Spot, "XBT/USD")]
 fn test_crawl_l2_snapshot(market_type: MarketType, symbol: &str) {
-    gen_test_snapshot_code!(
+    test_one_symbol!(
         crawl_l2_snapshot,
         EXCHANGE_NAME,
         market_type,
@@ -59,7 +54,7 @@ fn test_crawl_l2_snapshot(market_type: MarketType, symbol: &str) {
 
 #[test_case(MarketType::Spot)]
 fn test_crawl_l2_snapshot_without_symbol(market_type: MarketType) {
-    gen_test_snapshot_without_symbol_code!(
+    test_all_symbols!(
         crawl_l2_snapshot,
         EXCHANGE_NAME,
         market_type,
@@ -69,7 +64,7 @@ fn test_crawl_l2_snapshot_without_symbol(market_type: MarketType) {
 
 #[test_case(MarketType::Spot, "XBT/USD")]
 fn test_crawl_ticker(market_type: MarketType, symbol: &str) {
-    gen_test_code!(
+    test_one_symbol!(
         crawl_ticker,
         EXCHANGE_NAME,
         market_type,
